function arccos(x:real): real;
begin
    if abs(x) > 0.5 then
        if x>0 then arccos:= arctan( sqrt(1-sqr(x))/x )
        else arccos:= arctan( sqrt(1-sqr(x))/x ) + PI
    else arccos:= PI/2-arctan( x/sqrt(1-sqr(x)))
end;

function arcsin(x:real): real;
begin
    arcsin:= PI/2 - arccos(x)
end;

function arctan2( x, y: real): real;
var alfa: real;
begin
    if abs(y) < TOL then alfa:= PI/2 - arctan(y/x)
    else alfa:= arctan(x/y);
    if y<0 then
        if x<0 then alfa:= alfa - PI
        else alfa:= alfa + PI;
    arctan2:= alfa
end;

function tan(alfa:real): real;
begin
    tan:= sin(alfa)/cos(alfa)
end;

procedure BOLnaarXYZ(phi, psi: real; var p: punt3);
var cosPsi: real;
begin
    cosPsi:= cos(psi);
    p.x:= sin(phi)*cosPsi;
    p.y:= sin(psi);
    p.z:= cos(phi)*cosPsi
end;

procedure XYZnaarBOL(p: punt3; var phi, psi: real);
var r: real;
begin
    psi:= arcsin(p.y);
    r:= sqrt( sqr(p.x) + sqr(p.z) );
    if r<TOL then phi:=0
    else begin
        phi:= arccos(p.z/r);
        if p.x<0 then phi:= -phi
    end
end;

procedure maakGonio( alfa: real; var gAlfa: goniotype);
begin
    gAlfa.sinus:= sin(alfa);
    gAlfa.cosinus:= cos(alfa)
end;

procedure asRotatie( gAlfa: goniotype; var as1, as2: real);
var as: real;
begin
     as:= gAlfa.cosinus*as1 -  gAlfa.sinus*as2;
    as2:=  gAlfa.sinus*as1  + gAlfa.cosinus*as2;
    as1:= as
end;

procedure uitprodukt( p1, p2: punt3; var  q: punt3);
begin
    q.x:= p1.y * p2.z  -  p1.z * p2.y;
    q.y:= p1.z * p2.x  -  p1.x * p2.z;
    q.z:= p1.x * p2.y  -  p1.y * p2.x
end;

function inprodukt( p, q: punt3): real;
begin
    inprodukt:= p.x * q.x  +  p.y * q.y  +  p.z * q.z
end;

function lengte( p: punt3): real;
begin
    lengte:= sqrt( sqr(p.x) + sqr(p.y) + sqr(p.z) )
end;

function det3( p, q, r: punt3): real;
var t: punt3;
begin
    uitprodukt(p, q, t);
    det3:= inprodukt(t , r)
end;

function afstand( p, q: punt3): real;
begin
    afstand:= sqrt( sqr(p.x-q.x) + sqr(p.y-q.y) + sqr(p.z-q.z) )
end;

procedure linkomb( m: real; p: punt3; n: real; q: punt3; var r: punt3);
begin
    r.x:= m * p.x  +  n * q.x;
    r.y:= m * p.y  +  n * q.y;
    r.z:= m * p.z  +  n * q.z
end;

procedure konstPunt( k: real; var p: punt3);
begin
    p.x:=  k * p.x;
    p.y:=  k * p.y;
    p.z:=  k * p.z
end;

procedure midden( p, q: punt3; var m: punt3);
begin
    linkomb( 0.5, p, 0.5, q, m);
    konstPunt( 1/lengte(m), m)
end;

function maximum( a, b: real): real;
begin
    if a>b then maximum:= a
           else maximum:= b
end;

function minimum( a, b: real): real;
begin
    if a<b then minimum:= a
           else minimum:= b
end;

function hoek3( p, q, r: punt3): real;
var
    v, w: punt3;
      h3: real;
begin
    linkomb( 1, q, -1, p, v);    { v:= q-p }
    linkomb( 1, r, -1, q, w);    { w:= r-q }
    h3:= arccos( inprodukt( v, w) / (lengte(v) * lengte(w) ) );
    if det3( p, q, r) < 0 then hoek3:= -h3
                          else hoek3:= h3
end;

function hoekVerschil( hoek1, hoek2: real): real;
var h: real;
begin
    h:= abs( hoek1 - hoek2 );
    if h >= PI then hoekVerschil:= 2*PI - h
               else hoekVerschil:= h
end;

procedure tussenPunt( e0, p0, p1: punt3; var e1: punt3; afst1, L: real);
{         afstand( e0, p0) <  L                              }
{ afst1 = afstand( e0, p1) >= L                              }
{ e1 ligt op het lijnstuk p0-p1, zodat afstand( e0, e1) = L. }

var
                q0, q1: punt3;
    k, D, cosAlfa, len: real;

begin
    linkomb( 1, p1, -1, e0, q0);    { q0:= p1 - e0 }
    linkomb( 1, p1, -1, p0, q1);    { q1:= p1 - p0 }
    len:= afstand( p0, p1);

    { toepassen van de cosinusregel:                          }
    { L^2  =  c^2  +  afst1^2  -  2 * c * afst1 * cos(alfa);  }
    { dit is een tweedegraadsvergelijking met c.              }
    { alfa: hoek p0,p1,e0.                                    }
    {    c: afstand van p1 tot bol met straal L rond e0.      }

    cosAlfa:= inprodukt( q0, q1) / (afst1*len);
    D:= sqrt( sqr(afst1) * ( sqr(cosAlfa) -1 ) + sqr(L) );
                                    { D: sqrt(Diskriminant)/2 }

    k:= (afst1*cosAlfa - D) / len;     { oplossing: c = k*len }

    linkomb( k, p0, 1-k, p1, e1);
    konstPunt( 1/lengte(e1), e1)
end;    { tussenPunt }

procedure aardePuntMee( var p: punt3);
begin
    asRotatie( gPhiMid, p.x, p.z);
    asRotatie( gPsiMid, p.y, p.z)
end;    { aardePuntMee }
