{$N+}

UNIT gritmath;

INTERFACE

(* e.h. dooijes / april 2007 *)
(* derived from grit *)
(* identiek aan dxymath / okt 2007 *)

type VEC3 =array[1..3] of real;
     MAT33=array[1..3,1..3] of real;
     VEC4 =array[1..4] of real;
     MAT44=array[1..4,1..4] of real;

function  Det3(a,b,c: VEC3): real;
function  VVprod3(a,b: VEC3): real;
procedure MMprod3(m1,m2: MAT33;var m: MAT33);
procedure MVprod3(m: MAT33;u: VEC3;var v: VEC3);
procedure Rot2D(theta: real;var m: MAT33);
procedure Transl2D(tx,ty: real;var m: MAT33);
procedure Scale2D(sx,sy: real;var m: MAT33);
function  Det4(a,b,c,d: VEC4): real;
function  VVprod4(a,b: VEC4): real;
procedure MMprod4(m1,m2: MAT44;var m: MAT44);
procedure MVprod4(m: MAT44;u: VEC4;var v: VEC4);
procedure RotX3D(theta: real;var m: MAT44);
procedure RotY3D(theta: real;var m: MAT44);
procedure RotZ3D(theta: real;var m: MAT44);
procedure Transl3D(tx,ty,tz: real;var m: MAT44);
procedure Scale3D(sx,sy,sz: real;var m: MAT44);
procedure RtoL(var m: MAT44);
procedure EyetoClip(s,d,f: real;var m: MAT44);
procedure Plane(p,q,r: VEC4;var a: VEC4);


IMPLEMENTATION


(************** VECTOR & MATRIX ROUTINES ********************)

function det33(a11,a12,a13,a21,a22,a23,a31,a32,a33:real):real;

  function det22(a11,a12,a21,a22:real):real;
  begin
    det22:=a11*a22-a21*a12
  end;

begin
  det33:=a11*det22(a22,a23,a32,a33)
        -a12*det22(a21,a23,a31,a33)
        +a13*det22(a21,a22,a31,a32)
end;

function Det3;
begin
  Det3:=det33(a[1],b[1],c[1],a[2],b[2],c[2],a[3],b[3],c[3])
end;

procedure imat3(var m:MAT33);
var r,k:integer;
begin
  for r:=1 to 3 do
    for k:=1 to 3 do
      if r=k then m[r,k]:=1
             else m[r,k]:=0
end;

function VVprod3;
var i:integer;
    prod:real;
begin
  prod:=0;
  for i:=1 to 3 do prod:=prod+a[i]*b[i];
  VVprod3:=prod
end;

procedure MMprod3;
var i,r,k:integer;
begin
  for r:=1 to 3 do
    for k:=1 to 3 do
      begin
        m[r,k]:=0;
        for i:=1 to 3 do m[r,k]:=m[r,k]+m1[r,i]*m2[i,k]
      end
end;

procedure Rot2D;
var c,s:real;
begin
  s:=sin(theta); c:=cos(theta);
  imat3(m);
  m[1,1]:= c; m[2,2]:=c;
  m[1,2]:=-s; m[2,1]:=s
end;

procedure Transl2D;
begin
  imat3(m);
  m[1,3]:=tx;
  m[2,3]:=ty
end;

procedure Scale2D;
begin
  imat3(m);
  m[1,1]:=sx; m[2,2]:=sy
end;

procedure MVprod3;
var i,j:integer;
begin
  for i:=1 to 3 do
    begin
      v[i]:=0;
      for j:=1 to 3 do v[i]:=v[i]+m[i,j]*u[j]
    end
end;

function Det4;
begin
  Det4:=a[1]*det33(b[2],c[2],d[2],b[3],c[3],d[3],b[4],c[4],d[4])
       -a[2]*det33(b[1],c[1],d[1],b[3],c[3],d[3],b[4],c[4],d[4])
       +a[3]*det33(b[1],c[1],d[1],b[2],c[2],d[2],b[4],c[4],d[4])
       -a[4]*det33(b[1],c[1],d[1],b[2],c[2],d[2],b[3],c[3],d[3])
end;

procedure imat4(var m:MAT44);
var r,k:integer;
begin
  for r:=1 to 4 do
    for k:=1 to 4 do
      if r=k then m[r,k]:=1
             else m[r,k]:=0
end;

function VVprod4;
var i:integer;
    prod:real;
begin
  prod:=0;
  for i:=1 to 4 do prod:=prod+a[i]*b[i];
  VVprod4:=prod
end;

procedure MMprod4;
var i,r,k:integer;
begin
  for r:=1 to 4 do
    for k:=1 to 4 do
      begin
        m[r,k]:=0;
        for i:=1 to 4 do m[r,k]:=m[r,k]+m1[r,i]*m2[i,k]
      end
end;

procedure RotX3D;
var c,s:real;
begin
  s:=sin(theta); c:=cos(theta);
  imat4(m);
  m[2,2]:=c; m[2,3]:=-s;
  m[3,2]:=s; m[3,3]:= c
end;

procedure RotY3D;
var c,s:real;
begin
  s:=sin(theta); c:=cos(theta);
  imat4(m);
  m[1,1]:= c;  m[1,3]:=s;
  m[3,1]:=-s; m[3,3]:=c
end;

procedure RotZ3D;
var c,s:real;
begin
  s:=sin(theta); c:=cos(theta);
  imat4(m);
  m[1,1]:=c; m[1,2]:=-s;
  m[2,1]:=s; m[2,2]:= c
end;

procedure Transl3D;
begin
  imat4(m);
  m[1,4]:=tx; m[2,4]:=ty; m[3,4]:=tz
end;

procedure Scale3D;
begin
  imat4(m);
  m[1,1]:=sx; m[2,2]:=sy; m[3,3]:=sz
end;

procedure MVprod4;
var i,j:integer;
begin
  for i:=1 to 4 do
    begin
      v[i]:=0;
      for j:=1 to 4 do v[i]:=v[i]+m[i,j]*u[j]
    end
end;

procedure RtoL;
begin
  imat4(m);
  m[3,3]:=-1
end;

procedure EyetoClip;
begin
  imat4(m);
  m[3,3]:= s/(d*(1.0-d/f));
  m[3,4]:=-s/(1.0-d/f);
  m[4,3]:= s/d;
  m[4,4]:=0
end;

procedure Plane;
begin
  a[1]:= det33(p[2],q[2],r[2],p[3],q[3],r[3],p[4],q[4],r[4]);
  a[2]:=-det33(p[1],q[1],r[1],p[3],q[3],r[3],p[4],q[4],r[4]);
  a[3]:= det33(p[1],q[1],r[1],p[2],q[2],r[2],p[4],q[4],r[4]);
  a[4]:=-det33(p[1],q[1],r[1],p[2],q[2],r[2],p[3],q[3],r[3]);
end;

END.

